@@ -11,14 +11,14 @@ use alloc::string::String;
 #[cfg(feature = "std")]
 use alloc::format;
 
-/// Similar to [Iter](core::slice::Iter), but specifically implemented with StaticVecs in mind.
+/// Similar to [`Iter`](core::slice::Iter), but specifically implemented with StaticVecs in mind.
 pub struct StaticVecIterConst<'a, T: 'a> {
   pub(crate) start: *const T,
   pub(crate) end: *const T,
   pub(crate) marker: PhantomData<&'a T>,
 }
 
-/// Similar to [IterMut](core::slice::IterMut), but specifically implemented with StaticVecs in
+/// Similar to [`IterMut`](core::slice::IterMut), but specifically implemented with StaticVecs in
 /// mind.
 pub struct StaticVecIterMut<'a, T: 'a> {
   pub(crate) start: *mut T,
@@ -40,7 +40,8 @@ impl<'a, T: 'a> StaticVecIterConst<'a, T> {
     unsafe {
       format!(
         "Current value of element at `start`: {:?}\nCurrent value of element at `end`: {:?}",
-        *self.start, *self.end
+        *self.start,
+        *self.end.offset(-1)
       )
     }
   }
@@ -143,7 +144,8 @@ impl<'a, T: 'a> StaticVecIterMut<'a, T> {
     unsafe {
       format!(
         "Current value of element at `start`: {:?}\nCurrent value of element at `end`: {:?}",
-        *self.start, *self.end
+        *self.start,
+        *self.end.offset(-1)
       )
     }
   }
