@@ -40,7 +40,7 @@ mod trait_impls;
 #[doc(hidden)]
 pub mod utils;
 
-/// A [Vec](alloc::vec::Vec)-like struct (mostly directly API-compatible where it can be)
+/// A [`Vec`](alloc::vec::Vec)-like struct (mostly directly API-compatible where it can be)
 /// implemented with const generics around an array of fixed `N` capacity.
 pub struct StaticVec<T, const N: usize> {
   data: [MaybeUninit<T>; N],
@@ -61,7 +61,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   /// which can be either `&mut` or `&` as if it is `&mut` it will implicitly coerce to `&`.
   /// If the slice has a length greater than the StaticVec's declared capacity,
   /// any contents after that point are ignored.
-  /// Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues.
   #[inline]
   pub fn new_from_slice(values: &[T]) -> Self
   where T: Copy {
@@ -85,8 +85,8 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   /// any contents after that point are ignored.
   /// The `N2` parameter does not need to be provided explicitly, and can be inferred from the array
   /// itself. This function does *not* leak memory, as any ignored extra elements in the source
-  /// array are explicitly dropped with [drop_in_place](core::ptr::drop_in_place) before
-  /// [forget](core::mem::forget) is called on it.
+  /// array are explicitly dropped with [`drop_in_place`](core::ptr::drop_in_place) before
+  /// [`forget`](core::mem::forget) is called on it.
   #[inline]
   pub fn new_from_array<const N2: usize>(mut values: [T; N2]) -> Self {
     Self {
@@ -139,11 +139,11 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   }
 
   /// Returns a new StaticVec instance filled with the return value of an initializer function.
-  /// Unlike for [filled_with](crate::StaticVec::filled_with), the initializer function in
+  /// Unlike for [`filled_with`](crate::StaticVec::filled_with), the initializer function in
   /// this case must take a single usize variable as an input parameter, which will be called
   /// with the current index of the `0..N` loop that
-  /// [filled_with_by_index](crate::StaticVec::filled_with_by_index) is implemented with internally.
-  /// The length field of the newly created StaticVec will be equal to its capacity.
+  /// [`filled_with_by_index`](crate::StaticVec::filled_with_by_index) is implemented with
+  /// internally. The length field of the newly created StaticVec will be equal to its capacity.
   ///
   /// Example usage:
   /// ```
@@ -168,9 +168,10 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   }
 
   /// Returns the current length of the StaticVec.
-  /// Just as for a normal [Vec](alloc::vec::Vec), this means the number of elements that
-  /// have been added to it with `push`, `insert`, etc. except in the case
-  /// that it has been set directly with the unsafe `set_len` function.
+  /// Just as for a normal [`Vec`](alloc::vec::Vec), this means the number of elements that
+  /// have been added to it with [`push`](crate::StaticVec::push),
+  /// [`insert`](crate::StaticVec::insert), etc. except in the case that it has been set directly
+  /// with the unsafe [`set_len`](crate::StaticVec::set_len) function.
   #[inline(always)]
   pub const fn len(&self) -> usize {
     self.length
@@ -184,24 +185,24 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     N
   }
 
-  /// Does the same thing as [capacity](crate::StaticVec::capacity), but as an associated
+  /// Does the same thing as [`capacity`](crate::StaticVec::capacity), but as an associated
   /// function rather than a method.
   #[inline(always)]
   pub const fn cap() -> usize {
     N
   }
 
-  /// Serves the same purpose as [capacity](crate::StaticVec::capacity), but as an associated
+  /// Serves the same purpose as [`capacity`](crate::StaticVec::capacity), but as an associated
   /// constant rather than a method.
   pub const CAPACITY: usize = N;
 
-  /// Returns the remaining capacity of the `StaticVec`.
+  /// Returns the remaining capacity of the StaticVec.
   #[inline(always)]
   pub const fn remaining_capacity(&self) -> usize {
     N - self.length
   }
 
-  /// Directly sets the `length` field of the StaticVec to `new_len`. Useful if you intend
+  /// Directly sets the length field of the StaticVec to `new_len`. Useful if you intend
   /// to write to it solely element-wise, but marked unsafe due to how it creates
   /// the potential for reading from unitialized memory later on.
   #[inline(always)]
@@ -271,16 +272,14 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   /// ensure that is the case, so this function is marked `unsafe` and should
   /// be used with caution only when performance is absolutely paramount.
   ///
-  /// Note that, unlike [`slice::get_unchecked`], this method only supports
+  /// Note that unlike [`slice::get_unchecked`](https://doc.rust-lang.org/std/primitive.slice.html#method.get_unchecked_mut), this method only supports
   /// accessing individual elements via `usize`; it cannot also produce
   /// subslices. To unsafely get a subslice without a bounds check, use
-  /// `vec.as_slice().get_unchecked(a..b)`.
+  /// `self.as_slice().get_unchecked(a..b)`.
   ///
   /// # Safety
   ///
   /// It is up to the caller to ensure that `index` is within the appropriate bounds.
-  ///
-  /// [`slice::get_unchecked`]: https://doc.rust-lang.org/nightly/std/primitive.slice.html#method.get_unchecked
   #[inline(always)]
   pub unsafe fn get_unchecked(&self, index: usize) -> &T {
     debug_assert!(
@@ -298,7 +297,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   /// be used with caution only when performance is absolutely paramount.
   ///
   /// The same differences between this method and the slice method of the same name
-  /// apply as do for [get_unchecked](crate::StaticVec::get_unchecked).
+  /// apply as do for [`get_unchecked`](crate::StaticVec::get_unchecked).
   ///
   /// # Safety
   ///
@@ -351,7 +350,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     Err("Insufficient remaining capacity!")
   }
 
-  /// Push a value to the end of this `StaticVec`. Panics if the collection is
+  /// Push a value to the end of this StaticVec. Panics if the collection is
   /// full; that is, if `self.len() == self.capacity()`.
   #[inline(always)]
   pub fn push(&mut self, value: T) {
@@ -511,7 +510,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     self.length = 0;
   }
 
-  /// Returns a [StaticVecIterConst](crate::iterators::StaticVecIterConst) over the StaticVec's
+  /// Returns a [`StaticVecIterConst`](crate::iterators::StaticVecIterConst) over the StaticVec's
   /// inhabited area.
   #[inline(always)]
   pub fn iter<'a>(&'a self) -> StaticVecIterConst<'a, T> {
@@ -525,7 +524,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     }
   }
 
-  /// Returns a [StaticVecIterMut](crate::iterators::StaticVecIterMut) over the StaticVec's
+  /// Returns a [`StaticVecIterMut`](crate::iterators::StaticVecIterMut) over the StaticVec's
   /// inhabited area.
   #[inline(always)]
   pub fn iter_mut<'a>(&'a mut self) -> StaticVecIterMut<'a, T> {
@@ -541,8 +540,8 @@ impl<T, const N: usize> StaticVec<T, { N }> {
 
   /// Returns a separate, stable-sorted StaticVec of the contents of the
   /// StaticVec's inhabited area without modifying the original data.
-  /// Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues,
-  /// and [Ord](core::cmp::Ord) to make the sorting possible.
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues,
+  /// and [`Ord`](core::cmp::Ord) to make the sorting possible.
   #[cfg(feature = "std")]
   #[doc(cfg(feature = "std"))]
   #[inline]
@@ -555,8 +554,8 @@ impl<T, const N: usize> StaticVec<T, { N }> {
 
   /// Returns a separate, unstable-sorted StaticVec of the contents of the
   /// StaticVec's inhabited area without modifying the original data.
-  /// Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues,
-  /// and [Ord](core::cmp::Ord) to make the sorting possible.
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues,
+  /// and [`Ord`](core::cmp::Ord) to make the sorting possible.
   #[inline]
   pub fn sorted_unstable(&self) -> Self
   where T: Copy + Ord {
@@ -567,7 +566,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
 
   /// Returns a separate, reversed StaticVec of the contents of the StaticVec's
   /// inhabited area without modifying the original data.
-  /// Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues.
   #[inline]
   pub fn reversed(&self) -> Self
   where T: Copy {
@@ -588,7 +587,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   /// Copies and appends all elements, if any, of a slice (which can also be `&mut` as it will
   /// coerce implicitly to `&`) to the StaticVec. If the slice has a length greater than the
   /// StaticVec's remaining capacity, any contents after that point are ignored.
-  /// Locally requires that `T` implements [Copy](core::marker::Copy) to avoid soundness issues.
+  /// Locally requires that `T` implements [`Copy`](core::marker::Copy) to avoid soundness issues.
   #[inline(always)]
   pub fn extend_from_slice(&mut self, other: &[T])
   where T: Copy {
@@ -621,12 +620,36 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     Ok(())
   }
 
-  /// Returns a [Vec](alloc::vec::Vec) containing the contents of the StaticVec instance.
-  /// The returned [Vec](alloc::vec::Vec) will initially have the same value for
-  /// [len](alloc::vec::Vec::len) and [capacity](alloc::vec::Vec::capacity) as the source StaticVec.
-  /// Note that while using this function does *not* consume the source StaticVec in the sense of
-  /// rendering it completely inaccessible / unusable, it *does* empty it (that is, it will have
-  /// no contents and a `length` of 0 afterwards.)
+  /// Appends `self.remaining_capacity()` (or as many as available) items from
+  /// `other` to `self`. The appended items (if any) will longer exist in `other` afterwards,
+  /// as `other`'s `length` field will be adjusted to indicate. The `N2` parameter does
+  /// not need to be provided explicitly, and can be inferred directly from the constant `N`
+  /// constraint of `other` (which may or may not be the same as the `N` constraint of `self`.)
+  #[inline]
+  pub fn append<const N2: usize>(&mut self, other: &mut StaticVec<T, { N2 }>) {
+    let item_count = self.remaining_capacity().min(other.length);
+    if item_count > 0 {
+      let other_new_length = other.length - item_count;
+      unsafe {
+        self
+          .as_mut_ptr()
+          .add(self.length)
+          .copy_from_nonoverlapping(other.as_ptr(), item_count);
+        other
+          .as_mut_ptr()
+          .copy_from(other.as_ptr().add(item_count), other_new_length);
+      }
+      other.length = other_new_length;
+      self.length += item_count;
+    }
+  }
+
+  /// Returns a [`Vec`](alloc::vec::Vec) containing the contents of the StaticVec instance.
+  /// The returned [`Vec`](alloc::vec::Vec) will initially have the same value for
+  /// [`len`](alloc::vec::Vec::len) and [`capacity`](alloc::vec::Vec::capacity) as the source
+  /// StaticVec. Note that while using this function does *not* consume the source StaticVec in
+  /// the sense of rendering it completely inaccessible / unusable, it *does* empty it (that is,
+  /// it will have no contents and a length of 0 afterwards.)
   #[cfg(feature = "std")]
   #[doc(cfg(feature = "std"))]
   #[inline(always)]
@@ -661,20 +684,22 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     };
     assert!(start <= end && end <= self.length);
     let res_length = end - start;
-    let mut res = Self::new_data_uninit();
-    unsafe {
-      self
-        .as_ptr()
-        .add(start)
-        .copy_to_nonoverlapping(res.as_mut_ptr() as *mut T, res_length);
-      self
-        .as_ptr()
-        .add(end)
-        .copy_to(self.as_mut_ptr().add(start), self.length - end);
-    }
-    self.length -= res_length;
     Self {
-      data: unsafe { res.assume_init() },
+      data: {
+        let mut res = Self::new_data_uninit();
+        unsafe {
+          self
+            .as_ptr()
+            .add(start)
+            .copy_to_nonoverlapping(res.as_mut_ptr() as *mut T, res_length);
+          self
+            .as_ptr()
+            .add(end)
+            .copy_to(self.as_mut_ptr().add(start), self.length - end);
+          self.length -= res_length;
+          res.assume_init()
+        }
+      },
       length: res_length,
     }
   }
@@ -720,10 +745,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
       let old_length = self.length;
       self.length = length;
       unsafe {
-        ptr::drop_in_place(
-          &mut *(self.data.get_unchecked_mut(length..old_length) as *mut [MaybeUninit<T>]
-            as *mut [T]),
-        );
+        ptr::drop_in_place(self.as_mut_slice().get_unchecked_mut(length..old_length));
       }
     }
   }
@@ -735,16 +757,16 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   pub fn split_off(&mut self, at: usize) -> Self {
     assert!(at <= self.length);
     let split_length = self.length - at;
-    let mut split = Self::new_data_uninit();
     self.length = at;
-    unsafe {
-      self
-        .as_ptr()
-        .add(at)
-        .copy_to_nonoverlapping(split.as_mut_ptr() as *mut T, split_length);
-    }
     Self {
-      data: unsafe { split.assume_init() },
+      data: unsafe {
+        let mut split = Self::new_data_uninit();
+        self
+          .as_ptr()
+          .add(at)
+          .copy_to_nonoverlapping(split.as_mut_ptr() as *mut T, split_length);
+        split.assume_init()
+      },
       length: split_length,
     }
   }
@@ -763,7 +785,7 @@ impl<T, const N: usize> StaticVec<T, { N }> {
   }
 
   /// Removes consecutive repeated elements in the StaticVec according to the
-  /// locally required [PartialEq](core::cmp::PartialEq) trait implementation for `T`.
+  /// locally required [`PartialEq`](core::cmp::PartialEq) trait implementation for `T`.
   #[inline(always)]
   pub fn dedup(&mut self)
   where T: PartialEq {
@@ -782,28 +804,6 @@ impl<T, const N: usize> StaticVec<T, { N }> {
     self.dedup_by(|a, b| key(a) == key(b))
   }
 
-  #[doc(hidden)]
-  #[inline(always)]
-  pub(crate) unsafe fn copy_length_to(&self, dest: *mut MaybeUninit<Self>) {
-    // An internal convenience function to copy a usize worth of bytes from
-    // `self` to `dest`, representing the space taken up by `self.length` but not necessarily the
-    // actual value of `self.length`. This function should *only* be used in conjunction with
-    // `copy_items_to`, because they are both written in terms of number of bytes copied as
-    // opposed to actual specific field accesses.
-    (self as *const Self as *const usize).copy_to_nonoverlapping(dest as *mut usize, 1);
-  }
-
-  #[doc(hidden)]
-  #[inline(always)]
-  pub(crate) unsafe fn copy_items_to(&self, dest: *mut MaybeUninit<Self>) {
-    // Copies `self.length` worth of items to `dest`, in bytes. Like `copy_length_to`, this function
-    // is only concerned with the *amount* of data being written, not field layouts, and so
-    // should never be used except in conjunction with it when initializing a StaticVec to be
-    // returned from something or assigned to something.
-    ((self as *const Self as *const usize).offset(1) as *const T)
-      .copy_to_nonoverlapping((dest as *mut usize).offset(1) as *mut T, self.length);
-  }
-
   #[doc(hidden)]
   #[inline(always)]
   pub(crate) fn new_data() -> [MaybeUninit<T>; N] {
