@@ -5,7 +5,6 @@ use core::cmp::{Eq, Ord, Ordering, PartialEq};
 use core::fmt::{self, Debug, Formatter};
 use core::hash::{Hash, Hasher};
 use core::iter::FromIterator;
-use core::mem::MaybeUninit;
 use core::ops::{Deref, DerefMut, Index, IndexMut};
 use core::ptr;
 use core::slice::SliceIndex;
@@ -50,8 +49,8 @@ impl<T: Clone, const N: usize> Clone for StaticVec<T, { N }> {
   default fn clone(&self) -> Self {
     let mut res = Self::new();
     for item in self {
-      // Safety: res and self have the same type, so they're guaranteed to
-      // have the same capacity, and push_unchecked will never overflow.
+      // Safety: `self` has the same capacity of `res`, and `res` is
+      // empty, so all of these pushes are safe.
       unsafe {
         res.push_unchecked(item.clone());
       }
@@ -62,20 +61,19 @@ impl<T: Clone, const N: usize> Clone for StaticVec<T, { N }> {
   #[inline]
   default fn clone_from(&mut self, rhs: &Self) {
     self.truncate(rhs.length);
-    for i in 0..rhs.length {
-      if i < self.length {
-        // Safety: after the truncate, `self.len` <= `rhs.len`, which means that for
-        // every i in self, there is definitely an element at `rhs[i]`.
-        unsafe {
-          self.get_unchecked_mut(i).clone_from(rhs.get_unchecked(i));
-        }
-      } else {
-        // Safety: `i` < `rhs.length`, so `rhs.get_unchecked()` is safe. `i` starts at
-        // `self.length`, which is <= `rhs.length`, so there is always an available
-        // slot at `self[i]` to push into.
-        unsafe {
-          self.push_unchecked(rhs.get_unchecked(i).clone());
-        }
+    for i in 0..self.length {
+      // Safety: after the truncate, `self.len` <= `rhs.len`, which means that for
+      // every `i` in `self`, there is definitely an element at `rhs[i]`.
+      unsafe {
+        self.get_unchecked_mut(i).clone_from(rhs.get_unchecked(i));
+      }
+    }
+    for i in self.length..rhs.length {
+      // Safety: `i` < `rhs.length`, so `rhs.get_unchecked` is safe. `i` starts at
+      // `self.length`, which is <= `rhs.length`, so there is always an available
+      // slot at `self[i]` to push into.
+      unsafe {
+        self.push_unchecked(rhs.get_unchecked(i).clone());
       }
     }
   }
@@ -87,22 +85,18 @@ impl<T: Copy, const N: usize> Clone for StaticVec<T, { N }> {
     match self.length {
       // If `self` is empty, just return a new StaticVec.
       0 => Self::new(),
-      _ => {
-        // Create an uninitialized StaticVec (which allows the compiler to do better return value
-        // optimization when we call `assume_init` on it later, rather than fully instantiating it
-        // now.)
-        let mut res = MaybeUninit::<Self>::uninit();
-        // Copy a usize worth of bytes to `res` for `length`, and then `self.length` times
-        // `size_of::<T>` bytes to `res` for `data`. Note that the order we do this in does
-        // not actually matter, it just matters that the total number of bytes copied is exactly
-        // correct (which we can be confident is true by using the next two functions one
-        // after another.)
-        unsafe {
-          self.copy_length_to(&mut res);
-          self.copy_items_to(&mut res);
-          res.assume_init()
-        }
-      }
+      _ => Self {
+        data: {
+          let mut res = Self::new_data_uninit();
+          unsafe {
+            self
+              .as_ptr()
+              .copy_to_nonoverlapping(res.as_mut_ptr() as *mut T, self.length);
+            res.assume_init()
+          }
+        },
+        length: self.length,
+      },
     }
   }
 
@@ -166,7 +160,7 @@ impl<'a, T: 'a + Copy, const N: usize> Extend<&'a T> for StaticVec<T, { N }> {
 
 impl<T: Copy, const N: usize> From<&[T]> for StaticVec<T, { N }> {
   /// Creates a new StaticVec instance from the contents of `values`, using
-  /// [new_from_slice](crate::StaticVec::new_from_slice) internally.
+  /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
   fn from(values: &[T]) -> Self {
     Self::new_from_slice(values)
@@ -175,7 +169,7 @@ impl<T: Copy, const N: usize> From<&[T]> for StaticVec<T, { N }> {
 
 impl<T: Copy, const N: usize> From<&mut [T]> for StaticVec<T, { N }> {
   /// Creates a new StaticVec instance from the contents of `values`, using
-  /// [new_from_slice](crate::StaticVec::new_from_slice) internally.
+  /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
   fn from(values: &mut [T]) -> Self {
     Self::new_from_slice(values)
@@ -184,7 +178,7 @@ impl<T: Copy, const N: usize> From<&mut [T]> for StaticVec<T, { N }> {
 
 impl<T, const N1: usize, const N2: usize> From<[T; N1]> for StaticVec<T, { N2 }> {
   /// Creates a new StaticVec instance from the contents of `values`, using
-  /// [new_from_array](crate::StaticVec::new_from_array) internally.
+  /// [`new_from_array`](crate::StaticVec::new_from_array) internally.
   #[inline(always)]
   fn from(values: [T; N1]) -> Self {
     Self::new_from_array(values)
@@ -193,7 +187,7 @@ impl<T, const N1: usize, const N2: usize> From<[T; N1]> for StaticVec<T, { N2 }>
 
 impl<T: Copy, const N1: usize, const N2: usize> From<&[T; N1]> for StaticVec<T, { N2 }> {
   /// Creates a new StaticVec instance from the contents of `values`, using
-  /// [new_from_slice](crate::StaticVec::new_from_slice) internally.
+  /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
   fn from(values: &[T; N1]) -> Self {
     Self::new_from_slice(values)
@@ -202,7 +196,7 @@ impl<T: Copy, const N1: usize, const N2: usize> From<&[T; N1]> for StaticVec<T,
 
 impl<T: Copy, const N1: usize, const N2: usize> From<&mut [T; N1]> for StaticVec<T, { N2 }> {
   /// Creates a new StaticVec instance from the contents of `values`, using
-  /// [new_from_slice](crate::StaticVec::new_from_slice) internally.
+  /// [`new_from_slice`](crate::StaticVec::new_from_slice) internally.
   #[inline(always)]
   fn from(values: &mut [T; N1]) -> Self {
     Self::new_from_slice(values)
@@ -243,7 +237,7 @@ impl<T, I: SliceIndex<[T]>, const N: usize> IndexMut<I> for StaticVec<T, { N }>
 #[cfg(feature = "std")]
 #[doc(cfg(feature = "std"))]
 impl<T, const N: usize> Into<Vec<T>> for &mut StaticVec<T, { N }> {
-  /// Functionally equivalent to [into_vec](crate::StaticVec::into_vec).
+  /// Functionally equivalent to [`into_vec`](crate::StaticVec::into_vec).
   #[inline(always)]
   fn into(self) -> Vec<T> {
     self.into_vec()
@@ -253,7 +247,8 @@ impl<T, const N: usize> Into<Vec<T>> for &mut StaticVec<T, { N }> {
 impl<'a, T: 'a, const N: usize> IntoIterator for &'a StaticVec<T, { N }> {
   type IntoIter = StaticVecIterConst<'a, T>;
   type Item = &'a T;
-  /// Returns a `StaticVecIterConst` over the StaticVec's inhabited area.
+  /// Returns a [`StaticVecIterConst`](crate::iterators::StaticVecIterConst) over the StaticVec's
+  /// inhabited area.
   #[inline(always)]
   fn into_iter(self) -> Self::IntoIter {
     self.iter()
@@ -263,7 +258,8 @@ impl<'a, T: 'a, const N: usize> IntoIterator for &'a StaticVec<T, { N }> {
 impl<'a, T: 'a, const N: usize> IntoIterator for &'a mut StaticVec<T, { N }> {
   type IntoIter = StaticVecIterMut<'a, T>;
   type Item = &'a mut T;
-  /// Returns a `StaticVecIterMut` over the StaticVec's inhabited area.
+  /// Returns a [`StaticVecIterMut`](crate::iterators::StaticVecIterMut) over the StaticVec's
+  /// inhabited area.
   #[inline(always)]
   fn into_iter(self) -> Self::IntoIter {
     self.iter_mut()
@@ -308,11 +304,11 @@ impl_partial_ord_with_as_slice_against_slice!([T1], &mut StaticVec<T2, { N }>);
 impl_partial_ord_with_as_slice_against_slice!(&[T1], StaticVec<T2, { N }>);
 impl_partial_ord_with_as_slice_against_slice!(&mut [T1], StaticVec<T2, { N }>);
 
-/// Read from a [`StaticVec`]. This implementation reads from the `StaticVec`
+/// Read from a StaticVec. This implementation reads from the StaticVec
 /// by copying bytes into the destination buffers, then shifting the remaining
 /// bytes over. This might be inefficient for your needs; consider using
 /// [`Cursor`] or [`[T] as Read`][slice-read] for more efficient
-/// ways to read out of a `StaticVec` without mutating it.
+/// ways to read out of a StaticVec without mutating it.
 ///
 /// [`Cursor`]: https://doc.rust-lang.org/nightly/std/io/struct.Cursor.html
 /// [slice-read]: https://doc.rust-lang.org/nightly/std/primitive.slice.html#impl-Read]
@@ -379,7 +375,7 @@ impl<const N: usize> Read for StaticVec<u8, { N }> {
 
   #[inline]
   fn read_vectored(&mut self, bufs: &mut [IoSliceMut]) -> io::Result<usize> {
-    // Minimize copies: copy to each output buf in sequence, then shfit the
+    // Minimize copies: copy to each output buf in sequence, then shift the
     // internal data only once. This as opposed to calling `read` in a loop,
     // which shifts the inner data each time.
     let mut start_ptr = self.as_ptr();
@@ -393,7 +389,7 @@ impl<const N: usize> Read for StaticVec<u8, { N }> {
         break;
       }
 
-      // The number of bytes we'll be reading out of self
+      // The number of bytes we'll be reading out of self.
       let read_length = self.length.min(buf.len());
 
       // Safety: start_ptr is known to point to the array in self, which
