@@ -1,35 +1,17 @@
 use crate::StaticVec;
 use core::cmp::{Ordering, PartialOrd};
 use core::intrinsics;
-use core::mem::MaybeUninit;
 
-#[cfg(not(miri))]
-#[inline(always)]
-pub(crate) const fn distance_between<T>(dest: *const T, origin: *const T) -> usize {
-  unsafe {
-    if intrinsics::size_of::<T>() > 0 {
-      intrinsics::exact_div(
-        (dest as usize).wrapping_sub(origin as usize),
-        intrinsics::size_of::<T>(),
-      )
-    } else {
-      (dest as usize).wrapping_sub(origin as usize)
-    }
-  }
-}
-
-#[cfg(miri)]
 #[inline(always)]
 pub(crate) fn distance_between<T>(dest: *const T, origin: *const T) -> usize {
-  unsafe {
-    if intrinsics::size_of::<T>() > 0 {
+  match intrinsics::size_of::<T>() {
+    0 => (dest as usize).wrapping_sub(origin as usize),
+    _ => unsafe {
       intrinsics::exact_div(
         (dest as usize).wrapping_sub(origin as usize),
         intrinsics::size_of::<T>(),
       )
-    } else {
-      (dest as usize).wrapping_sub(origin as usize)
-    }
+    },
   }
 }
 
@@ -51,11 +33,11 @@ where T: Copy {
   StaticVec::<T, { COUNT }> {
     data: {
       unsafe {
-        let mut data: [MaybeUninit<T>; COUNT] = MaybeUninit::uninit_array();
+        let mut data = StaticVec::<T, { COUNT }>::new_data_uninit();
         for i in 0..COUNT {
-          data.get_unchecked_mut(i).write(value);
+          (data.as_mut_ptr() as *mut T).add(i).write(value);
         }
-        data
+        data.assume_init()
       }
     },
     length: COUNT,
