@@ -77,7 +77,7 @@ impl<'a> Drop for LifespanCountingInstance<'a> {
   }
 }
 
-#[derive(Debug)]
+#[derive(Debug, Eq, PartialEq)]
 struct Struct {
   s: &'static str,
 }
@@ -93,6 +93,39 @@ impl Drop for Struct {
 #[cfg(feature = "std")]
 use std::io::{IoSlice, Write};
 
+#[test]
+fn append() {
+  let mut a = staticvec![Struct { s: "A" }, Struct { s: "B" }, Struct { s: "C" }];
+  let mut b = staticvec![
+    Struct { s: "D" },
+    Struct { s: "E" },
+    Struct { s: "F" },
+    Struct { s: "G" }
+  ];
+  let mut c = StaticVec::<Struct, 6>::new();
+  c.append(&mut a);
+  c.append(&mut b);
+  assert_eq!(format!("{:?}", a), "[]");
+  assert_eq!(format!("{:?}", b), "[Struct { s: \"G\" }]");
+  assert_eq!(
+    c,
+    staticvec![
+      Struct { s: "A" },
+      Struct { s: "B" },
+      Struct { s: "C" },
+      Struct { s: "D" },
+      Struct { s: "E" },
+      Struct { s: "F" }
+    ]
+  );
+  let mut d = staticvec![12, 24];
+  let mut e = staticvec![1, 2, 3];
+  d.pop().unwrap();
+  d.append(&mut e);
+  assert_eq!(e, [2, 3]);
+  assert_eq!(d, [12, 1]);
+}
+
 #[test]
 fn as_mut_ptr() {
   let mut v = staticvec![1, 2, 3];
@@ -121,14 +154,12 @@ fn as_slice() {
 #[test]
 fn bounds_to_string() {
   let mut v = staticvec![1, 2, 3, 4];
-  let mut it = v.iter();
-  it.next_back();
+  let it = v.iter();
   assert_eq!(
     "Current value of element at `start`: 1\nCurrent value of element at `end`: 4",
     it.bounds_to_string()
   );
-  let mut itm = v.iter_mut();
-  itm.next_back();
+  let itm = v.iter_mut();
   assert_eq!(
     "Current value of element at `start`: 1\nCurrent value of element at `end`: 4",
     itm.bounds_to_string()
